/**
 * Runner of "ui" command.
 * @memberof module:apeman-ui/lib
 * @inner
 * @constructor UiRunner
 * @param {object} config - Runner configuration.
 */

"use strict";

var async = require('async'),
    path = require('path'),
    extend = require('extend'),
    UiLogger = require('./ui_logger');

/** @lends UiRunner */
function UiRunner() {
    var s = this;
    s.init.apply(s, arguments);
}

UiRunner.prototype = {
    /** @constructs UiRunner */
    init: function (config) {
        var s = this;
        extend(s, config);
        s.validate();
    },
    /**
     * Apemanfile instance.
     */
    apemanfile: null,
    /**
     * Run a ui.
     * @param {string} name - Name of ui to generate..
     * @param {function} callback - Callback when done.
     */
    run: function (name, callback) {
        var s = this,
            apemanfile = s.apemanfile;
        async.waterfall([
            function (callback) {
                apemanfile.find('$ui', name, callback);
            },
            function (ui, callback) {
                var logger = new UiLogger({
                    verbose: s.verbose
                });
                logger.printUiStart();
                async.eachSeries([].concat(ui), function (ui, callback) {
                    s.invokeUi(name, ui, callback);
                }, function (err) {
                    if (err) {
                        logger.printUiFailed(err);
                        process.exit(1);
                    } else {
                        logger.printUiDone();
                    }
                    callback(err);
                });
            }
        ], callback);
    },
    /**
     * Validate configuration.
     */
    validate: function () {
        var s = this;
        if (!s.apemanfile) {
            throw new Error('apemanfile is required.');
        }
    },
    /**
     * Perform ui invocation.
     */
    invokeUi: function (name, ui, callback) {
        var s = this;
        var context = s.invocationContext(name);
        async.eachSeries([].concat(ui), function (ui) {
            ui(context, callback);
        }, callback);
    },
    invocationContext: function (name) {
        var s = this;
        return {
            verbose: s.verbose,
            logger: new UiLogger({}),
            name: name,
            dest: path.resolve(name)
        };
    }

};

module.exports = UiRunner;